/* (c) David Alan Gilbert 1995 */
/* Program to generate the data for the MEMC page translation tables
   in a format which can go into the assembler */
#include <assert.h>
#include <stdio.h>

void OutputPage(unsigned int LogicalPage,unsigned int PhysicalPage,
                unsigned int ProtectionLevel) {
  unsigned int res;

  res=0x03800000; /* Point to MEMC address generation tables */

  assert(ProtectionLevel<4);
  assert(PhysicalPage<128);
  assert(LogicalPage<1024);

  res|=(ProtectionLevel<<8);
  res|=(PhysicalPage & 15)<<3;
  res|=(PhysicalPage & 16)>>4;
  res|=(PhysicalPage & 32)>>3;
  res|=(PhysicalPage & 64)>>5;

  res|=(LogicalPage & 255)<<15;
  res|=(LogicalPage & 768)<<2;

  printf("  .word 0x%08x  @ Physical page %d\n",res,PhysicalPage);
};

int main() {
  int phys,logic;

  printf("@ Automatically generated by mktranstable.c - don't change!\n");
  printf("  .text\n  .align\n  .global memctranstable\n");
  printf("memctranstable:\n");

  /* First map physical bottom 480K (=15 pages) to top of address space for video */
  for(phys=0,logic=1024-15;phys<15;phys++,logic++)
    OutputPage(logic,phys,0 /* all read write */);

  /* Use page 15 as zero page */
  OutputPage(0,15,0 /* All read write */);

  /* Now map the rest (=128-16=112 pages) to the 24MB boundary for kernel */
  for(phys=16,logic=768;phys<128;phys++,logic++)
    OutputPage(logic,phys,0 /* all read write */);


  printf("  .align\n");
  return 0;
}
