  .text
  .align

  .global _stext
@ On entry here we will be entered from address 0, what we need to do first
@ is jump into ROM (this should map RAM in)
_stext:
  ldr pc,[ pc,#_entryptr - . - 8]

_entryptr:
  .word __entry


__entry:
  bl setupvidc
  bl setupioc
  bl setupmemc
  bl copykerneldown
  bl callkernel

idle:
  b idle

@ ---------------------------------------------------------------------------
@ Setup the VIDC to something like Mode 31 (800x600x16 colours)
setupvidc:
  adr r3,vidcdata
  adr r4,vidcdata_end
  mov r5,#0x3400000 @ VIDC base
setupvidc_loop:
  ldr r2,[r3],#4
  str r2,[r5,#0]
  cmp r3,r4
  blt setupvidc_loop

  movs pc,lr

vidcdata:
  .word 0x00000000 @ palette entry 0 (black)
  .word 0x0400000f @ palette entry 1 (red)
  .word 0x080000f0 @ palette entry 2 (green)
  .word 0x0C0000ff @ palette entry 3 (yellow)
  .word 0x10000f00 @ palette entry 4 (blue)
  .word 0x14000f0f @ palette entry 5 (magenta)
  .word 0x18000ff0 @ palette entry 6 (cyan)
  .word 0x1C000fff @ palette entry 7 (white)
  .word 0x20000222 @ palette entry 8 (dark grey)
  .word 0x24000007 @ palette entry 1 (half red)
  .word 0x28000070 @ palette entry 2 (half green)
  .word 0x2C000077 @ palette entry 3 (half yellow)
  .word 0x30000700 @ palette entry 4 (half blue)
  .word 0x34000707 @ palette entry 5 (half magenta)
  .word 0x38000770 @ palette entry 6 (half cyan)
  .word 0x3C000777 @ palette entry 7 (half white)
  .word 0x40000555 @ border colour (greyish)
  .word 0x44000654 @ cursor colour 0 (?)
  .word 0x48000254 @ cursor colour 1 (?)
  .word 0x4c000314 @ cursor colour 2 (?)
  @ skip 50-5c (reserved) and 60-7c (stereo image)
  .word 0x805c8000 @ Horizontal cycle (740 pixel times per line)
  .word 0x84004000 @ Horizontal sync width (4 pixels)
  .word 0x88010000 @ Horizontal border start (17 pixels from start HSync)
  .word 0x8c014000 @ Horizontal display start (17 pixels from start HSync)
  .word 0x90515000 @ Horizontal display end (650 pixels from start HSync)
  .word 0x94522000 @ Horizontal border end (657 pixels from start HSync)
  .word 0x98000000 @ Horizontal cursor start (0 pixels from HSync)
  @ skip 9c - Horizontal interlace register
  .word 0xa0aec000 @ Vertical cycle (700 rasters per frame)
  .word 0xa4000000 @ Vertical sync width (1 raster)
  .word 0xa800c000 @ Vertical border start (4 rasters from sync start)
  .word 0xac00c000 @ Vertical display start (4 rasters from sync start)
  .word 0xb096c000 @ Vertical display end (604 rasters from sync start)
  .word 0xb49b0000 @ Vertical border end (621 rasters from sync start)
  .word 0xb8020000 @ Vertical cursor start (7 rasters from sync start)
  .word 0xbc020000 @ Vertical cursor end (7 rasters from sync start)
  .word 0xc00001f0 @ Sound frequency register (240us ~4K)
  @ skip c4-dc (reserved)
  .word 0xe000000b @ Control register, normal op, vertical sync, no interlace, 4 bpp, 24MHz
vidcdata_end:

@ ---------------------------------------------------------------------------
setupmemc:
  @ First setup all the control registers
  adr r3,memcdata_start
  adr r4,memcdata_end
setupmemc_loop1:
  ldr r5,[r3],#4
  strb r5,[r5,#0]
  cmp r3,r4
  blt setupmemc_loop1

  @ Now the page translation table (defined elsewhere)
  ldr r3,[ pc,#memctransptr - . - 8]
  @ its got 128 pages
  mov r4,#128
setupmemc_loop2:
  ldr r5,[r3],#4
  str r5,[r5,#0]
  subs r4,r4,#1
  bne setupmemc_loop2

  movs pc,lr

memcdata_start:
  .word 0x036e070c  @ Control Register (!OS Mode,video dma, continuous refresh,slow rom, 32K page - was 036e170c)
  .word 0x03600000  @ Vinit (at address 0)
  .word 0x03620000  @ Vstart (at address 0)
  .word 0x0365e000  @ Vend (at address 480K)
  .word 0x0367e000  @ Cinit (at address 480K)
  .word 0x0369e040  @ Sstart (at address 480K+256bytes)
  .word 0x036bfffc  @ Send (at address 512K (just below?)
  .word 0x036c0000  @ Swap sound buffer?
memcdata_end:

memctransptr:
  .word memctranstable
  .align
@ ---------------------------------------------------------------------------
@ Setup the ioc
setupioc:
  adr r3,iocdata_start
  adr r4,iocdata_end
setupioc_loop:
  ldmia r3!,{r1,r2}
  strb r2,[r1,#0]
  cmp r3,r4
  blt setupioc_loop

  movs pc,lr

iocdata_start:
  .word 0x03200000 @ control register
  .word 0x000000ff @ everything undriven
  .word 0x03200014 @ IRQ clear
  .word 0x0000007c @ clear timer and IR/IF ints
  .word 0x03200018 @ irq mask reg a
  .word 0x00000000 @ all disabled
  .word 0x03200028 @ irq mask reg b
  .word 0x00000000 @ all disabled
  .word 0x03200038 @ firq mask
  .word 0x00000000 @ all disabled
  .word 0x03200040 @ t0 latch low
  .word 0x000000ff
  .word 0x03200044 @ t0 latch high
  .word 0x000000ff @ largest value
  .word 0x03200050 @ t1 latch low
  .word 0x000000ff
  .word 0x03200054 @ t1 latch high
  .word 0x000000ff @ largest value
  .word 0x03200060 @ t2 latch low
  .word 0x000000ff
  .word 0x03200064 @ t2 latch high
  .word 0x000000ff @ largest value
  .word 0x03200070 @ t3 latch low
  .word 0x000000ff
  .word 0x03200074 @ t3 latch high
  .word 0x000000ff @ largest value
iocdata_end:

@ ---------------------------------------------------------------------------
@ exits with kernel length in r0
copykerneldown:
  ldr r0,[ pc,#kernlenptr - . - 8]
  ldr r0,[r0,#0]
  add r1,r0,#0x600    @ copy the parameters down which are straight after kernel
  mov r2,#0x3800000+512*1024   @ Kernel starts 512K into ROM image
  mov r3,#24*1024*1024              @ and thats where the kernel goes
copykerneldown_loop:
  ldr r4,[r2],#4
  str r4,[r3],#4
  subs r1,r1,#4
  bne copykerneldown_loop

  movs pc,lr

kernlenptr:
  .word kernlen  @ thats an address where the kernel length is
@ ---------------------------------------------------------------------------
@ entered with kernel len in r0
callkernel:
  mov r1,r0
  add r0,r1,#24*1024*1024   @ thats where the parameter set is

  mov r2,#24*1024*1024
  @ldr sp,[r2,#0x30]
  @mov pc,#24*1024*1024
  mov pc,#0x2080000
